<?php
// Own namespace
namespace CoreFramework\Filesystem\Pointer\Output;

// Import framework stuff
use CoreFramework\FileSystem\BaseFileIo;
use CoreFramework\Filesystem\Pointer\OutputPointer;
use CoreFramework\Generic\NullPointerException;

/**
 * A class for writing files
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2017 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FrameworkRawFileOutputPointer extends BaseFileIo implements OutputPointer {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Create a file pointer based on the given file. The file will also
	 * be verified here.
	 *
	 * @param	$fileName	The file name we shall pass to fopen()
	 * @param	$mode		The output mode ('w', 'a' are valid)
	 * @throws	FileIsEmptyException	If the provided file name is empty.
	 * @throws	FileIoException			If fopen() returns not a file resource
	 * @return	void
	 */
	public static final function createFrameworkRawFileOutputPointer ($fileName, $mode) {
		// Some pre-sanity checks...
		if (is_null($fileName)) {
			// No filename given
			throw new FileIsEmptyException(NULL, self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
		} // END - if

		// Try to open a handler
		$filePointer = @fopen($fileName, $mode);
		if ((is_null($filePointer)) || ($filePointer === false)) {
			// Something bad happend
			throw new FileIoException ($fileName, self::EXCEPTION_FILE_POINTER_INVALID);
		} // END - if

		// Create new instance
		$pointerInstance = new FrameworkRawFileOutputPointer();

		// Set file pointer and file name
		$pointerInstance->setPointer($filePointer);
		$pointerInstance->setFileName($fileName);

		// Return the instance
		return $pointerInstance;
	}

	/**
	 * Write data to a file pointer
	 *
	 * @param	$dataStream		The data stream we shall write to the file
	 * @return	mixed			Number of writes bytes or false on error
	 * @throws	NullPointerException	If the file pointer instance
	 *									is not set by setPointer()
	 * @throws	InvalidResourceException	If there is being set
	 *											an invalid file resource
	 */
	public function writeToFile ($dataStream) {
		if (is_null($this->getPointer())) {
			// Pointer not initialized
			throw new NullPointerException($this, self::EXCEPTION_IS_NULL_POINTER);
		} elseif (!is_resource($this->getPointer())) {
			// Pointer is not a valid resource!
			throw new InvalidResourceException($this, self::EXCEPTION_INVALID_RESOURCE);
		}

		// Write data to the file pointer and return written bytes
		return fwrite($this->getPointer(), $dataStream);
	}

	/**
	 * Analyzes entries in index file. This will count all found (and valid)
	 * entries, mark invalid as damaged and count gaps ("fragmentation"). If
	 * only gaps are found, the file is considered as "virgin" (no entries).
	 *
	 * @return	void
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function analyzeFile () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Writes at given position by seeking to it.
	 *
	 * @param	$seekPosition	Seek position in file
	 * @param	$data			Data to be written
	 * @return	mixed			Number of writes bytes or false on error
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function writeAtPosition ($seedPosition, $data) {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Advances to next "block" of bytes
	 *
	 * @return	void
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function next () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Checks wether the current entry is valid (not at the end of the file).
	 * This method will return true if an emptied (nulled) entry has been found.
	 *
	 * @return	$isValid	Whether the next entry is valid
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function valid () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Gets current seek position ("key").
	 *
	 * @return	$key	Current key in iteration
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function key () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

}
